import re
from lxml import etree
import logging
from pyocs.pyocs_request import PyocsRequest


class PyocsDemand:
    _ocs_link_Prefix = 'http://ocs.gz.cvte.cn/tv/Tasks/view/'
    _logger = logging.getLogger(__name__)

    product_name = "产品型号"
    chip_name = "主芯片"
    flash_size_name = "FlashSize"
    port_name = "子板型"
    ddr_name = "ddrSize"
    region_name = "区域"
    ciplus_name = "CI PLUS功能"
    pwm_name = "占空比"
    tuner_name = "高频头"
    title_name = "Title"
    customer_batch_code = "客户批号"
    customer_machine = "客户机型"
    other_app_soft = "其它应用软件"
    wifi_bluetooth_type = "WIFI与蓝牙"
    ci_type = "可选功能CI类型"
    wifi_module_info = "可选功能WIFI模块"
    chip_remark = "主芯片备注"
    customer_special_requirement = "客户特殊需求"

    SAMPLE_ORDER = "样品订单"
    FORMAL_ORDER = "正式订单"

    # 样品订单获取方案名依赖的是产品型号，因为会依据方案名维护配置仓库，防止出现多个相同的仓库，为与正式订单维持一致，需要做一层映射
    project_map_dict = {
        "MS358": "MSD358",
        "RD8503": "RDA8503",
    }

    def __init__(self, ocs_number):
        self.order_type = self.FORMAL_ORDER
        self.ocs_number = ocs_number
        self._logger.setLevel(level=logging.WARNING)  # 控制打印级别
        ocs_link = self._ocs_link_Prefix + ocs_number
        ocs_url_response = PyocsRequest().pyocs_request_get(ocs_link, False)
        if ocs_url_response.status_code == 302:
            sample_order_link = ocs_url_response.headers["Location"]
            if "sample_order_view" in sample_order_link:
                self.order_type = self.SAMPLE_ORDER
                ocs_url_response = PyocsRequest().pyocs_request_get(sample_order_link, False)
                self.html = etree.HTML(ocs_url_response.text)
        elif ocs_url_response.status_code == 200:
            self.html = etree.HTML(ocs_url_response.text)
        else:
            raise RuntimeError('OCS访问存在问题')

    def get_ocs_html(self):
        return self.html

    def get_ocs_number(self):
        return self.ocs_number

    def get_ocs_software_engineer(self):
        if self.order_type == self.FORMAL_ORDER:
            t_xpath = '//*[@id="basicinfo_region"]//*[@id="Task__sw_user_id_history"]/../span/text()'
            t_list = self.html.xpath(t_xpath)
            t_str = "".join(t_list[0]).strip() if t_list else ""  # 删除开头与结尾的空白符（包括'\n', '\r',  '\t',  ' ')
            self._logger.info("软件工程师：" + str(t_str))
            return t_str
        else:
            return ""

    def get_ocs_customer(self):
        if self.order_type == self.FORMAL_ORDER:
            t_xpath = '//table[@class="table table-striped table-bordered table-condensed txt_break"]' \
                        '//th[contains(text(),"客户")]/../td/text()'
            t_list = self.html.xpath(t_xpath)
            t_str = "".join(t_list[0]).strip() if t_list else ""  # 删除开头与结尾的空白符（包括'\n', '\r',  '\t',  ' ')
            self._logger.info("客户：" + str(t_str))
            return t_str
        elif self.order_type == self.SAMPLE_ORDER:
            return "CVTE样品"
        else:
            return ""

    def get_ocs_title(self):
        t_xpath = '//title/text()'
        t_list = self.html.xpath(t_xpath)
        t_str = "".join(t_list).strip() if t_list else ""  # 删除开头与结尾的空白符（包括'\n', '\r',  '\t',  ' ')
        self._logger.info("Title：" + str(t_str))
        return t_str

    def get_ocs_project_name(self):
        if self.order_type == self.FORMAL_ORDER:
            t_xpath = '//span[@id="RdDeptIdSpan"]/text()'
            t_list = self.html.xpath(t_xpath)
            t_str = "".join(t_list).strip() if t_list else ""
            return t_str
        elif self.order_type == self.SAMPLE_ORDER:
            # 兼容样品订单，只能使用产品型号确定方案名
            product_name = self.get_product_name()
            project_name = product_name.split('.')[1].rstrip('T').rstrip('S').rstrip('I')
            project_name = self.project_map_dict[project_name] if project_name in self.project_map_dict.keys() \
                else project_name
            return project_name
        else:
            return ""

    def get_product_name(self):
        product_xpath = \
            '//th[contains(text(),"产品型号")]/../td/text()'
        product_list = self.html.xpath(product_xpath)
        product_str = "".join(product_list).strip().strip('\r\n') if product_list else ""
        self._logger.info("产品型号：" + str(product_str))
        return product_str

    def get_region_name(self):
        if self.order_type == self.FORMAL_ORDER:
            region_xpath = '//*[@id="SW_DefaultCountry_Td"]/text()'
            region_list = self.html.xpath(region_xpath)
            region_str = "".join(region_list).strip() if region_list else ""
            region_str = "" if region_str == '无' else region_str
            self._logger.info("区域：" + str(region_str))
            return region_str
        elif self.order_type == self.SAMPLE_ORDER:
            region_xpath = '//th[text()="销售区域"]/../td/text()'
            region_list = self.html.xpath(region_xpath)
            region_str = "".join(region_list[0]).strip() if region_list else ""
            region_str = "" if region_str == '无' else region_str
            return region_str
        else:
            return ""

    def get_ci_plus(self):
        if self.order_type == self.FORMAL_ORDER:
            ciplus_xpath = \
                '//*[@id="orderinfo_sw"]//th[contains(text(),"CI PLUS功能")]/../td/text()'
            ciplus_list = self.html.xpath(ciplus_xpath)
            ciplus_str = "".join(ciplus_list).strip() if ciplus_list else ""
            ciplus_str = "" if ciplus_str == '无' else ciplus_str
            self._logger.info("CI+：" + str(ciplus_str))
            return ciplus_str
        elif self.order_type == self.SAMPLE_ORDER:
            ciplus_xpath = '//th[text()="CI PLUS功能"]/../td/text()'
            ciplus_list = self.html.xpath(ciplus_xpath)
            ciplus_str = "".join(ciplus_list[0]).strip() if ciplus_list else ""
            ciplus_str = "" if ciplus_str == '无' else ciplus_str
            return ciplus_str
        else:
            return ""

    def get_pwm(self):
        if self.order_type == self.FORMAL_ORDER:
            pwm_xpath = '//*[@id="SW_PowerCurrent_Td"]/text()'
            pwm_list = self.html.xpath(pwm_xpath)
            if pwm_list:
                pwm_str = "".join(pwm_list).strip()
                self._logger.info("占空比：" + pwm_str)
                if pwm_str == '无':
                    pwm = '100'
                else:
                    pwm_original = pwm_str.split('=')
                    pwm = pwm_original[1].strip(')').strip('%')
            else:
                pwm = '100'
            return pwm
        elif self.order_type == self.SAMPLE_ORDER:
            pwm_xpath = '//th[text()="电源规格及修改项"]/../td[@colspan="3"]/text()'
            pwm_list = self.html.xpath(pwm_xpath)
            pwm_str_origin = "".join(pwm_list).strip()
            pwm_pattern = re.compile("[0-9]*%")
            pwm_str_list = pwm_pattern.findall(pwm_str_origin)
            pwm_str = pwm_str_list[0].strip('%') if pwm_str_list else '100'
            if pwm_str.isdigit():
                return pwm_str
            else:
                raise RuntimeError('样品单占空比识别错误，请检查是否符合格式要求')
        else:
            return ""

    def get_customer_machine(self):
        if self.order_type == self.FORMAL_ORDER:
            product_xpath = \
                '//th[contains(text(),"客户机型")]/../td/text()'
            product_list = self.html.xpath(product_xpath)
            product_str = "".join(product_list).strip().strip('\r\n') if product_list else ""
            product_str = product_str if product_str else ""
            self._logger.info("客户机型：" + str(product_str))
            return product_str
        elif self.order_type == self.SAMPLE_ORDER:
            return ""
        else:
            return ""

    def get_other_app_software(self):
        if self.order_type == self.FORMAL_ORDER:
            app_xpath = '//*[@id="SW_OtherApplicationSoftware_Td"]/text()'
            app_list = self.html.xpath(app_xpath)
            app_str = "".join(app_list).strip() if app_list else ""
            app_str = "" if app_str == '无' else app_str
            self._logger.info("其它应用软件：" + str(app_str))
            return app_str
        elif self.order_type == self.SAMPLE_ORDER:
            app_xpath = '//th[text()="互联网电视牌照"]/../td/text()'
            app_list = self.html.xpath(app_xpath)
            app_str = "".join(app_list[0]).strip() if app_list else ""
            app_str = "" if app_str == '无' else app_str
            return app_str
        else:
            return ""

    def get_chip_name(self):
        chip_xpath = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"主芯片")]/../td/strong/text()'
        chip_list = self.html.xpath(chip_xpath)
        chip_str = "".join(chip_list).strip('\n').strip() if chip_list else ""
        self._logger.info("主芯片：" + chip_str)
        return chip_str

    def get_chip_remark(self):
        chip_ramark_xpath = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"主芯片")]/../td/p/em[contains(text(),"特殊描述")]/../text()'
        chip_remark_list = self.html.xpath(chip_ramark_xpath)
        chip_remark_str = "".join(chip_remark_list).strip('\n').strip() if chip_remark_list else ""
        self._logger.info("主芯片备注：" + chip_remark_str)
        return chip_remark_str

    def get_flash_size(self):
        flash_size_xpath = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"FlashSize")]/../td/strong/text()'
        flash_size_list = self.html.xpath(flash_size_xpath)
        if flash_size_list:
            flash_str_original = flash_size_list[0]
            flash_pattern = re.compile("[0-9][a-zA-Z]Byte")
            flash_size_list = flash_pattern.findall(flash_str_original)
            if flash_size_list:
                flash_size = flash_size_list[0]
                self._logger.info("flash size：" + flash_size)
                return flash_size
            else:
                return ""
        else:
            return ""

    def get_port_name(self):
        port_xpath = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"端子功能")]/../td/p/em[contains(text(),"通用备注")]/../text()'
        port_list = self.html.xpath(port_xpath)
        port_str_orinal = "".join(port_list).strip() if port_list else ""
        if port_str_orinal:
            port_str = port_str_orinal.strip().strip(':').split(':')[0]
            self._logger.info("子板型：" + str(port_str))
            return port_str
        else:
            return ""

    def get_ddr_size(self):
        ddr_str_original = ''
        ddr_xpath_0 = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"主芯片")]/../td//p/text()'
        ddr_list_0 = self.html.xpath(ddr_xpath_0)
        ddr_str_original += "".join(ddr_list_0) if ddr_list_0 else ''
        ddr_xpath_1 = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"External")]/../td/strong/text()'
        ddr_list_1 = self.html.xpath(ddr_xpath_1)
        ddr_str_original += "".join(ddr_list_1) if ddr_list_1 else ''
        ddr_xpath_2 = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"External")]/../td//p/text()'
        ddr_list_2 = self.html.xpath(ddr_xpath_2)
        ddr_str_original += "".join(ddr_list_2) if ddr_list_2 else ''
        # print(ddr_str_original)
        if ddr_str_original:
            ddr_pattern = re.compile("[0-9.]*[MG]B")
            ddr_str_list = ddr_pattern.findall(ddr_str_original)
            if ddr_str_list:
                ddr_str = ddr_str_list[-1]
            else:
                return ""
        else:
            ddr_str = ""
        self._logger.info("DDR：" + str(ddr_str))
        return ddr_str

    def get_tuner_type(self):
        tuner_xpath = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"高频头")]/../td/strong/text()'
        tuner_list = self.html.xpath(tuner_xpath)
        tuner_str = "".join(tuner_list) if tuner_list else ""
        self._logger.info("高频头：" + str(tuner_str))
        return tuner_str

    def get_customer_batch_code(self):
        product_xpath = \
            '//th[contains(text(),"客户批号")]/../td/text()'
        product_list = self.html.xpath(product_xpath)
        if product_list:
            product_str = product_list[0].strip().strip('\r\n')
            if product_str:
                self._logger.info("客户批号：" + product_str)
                return product_str
            else:
                return ""
        else:
            return ""

    def get_wifi_bluetooth(self):
        wifi_bluetooth_str = ''
        wifi_bluetooth_xpath = '//*[@id="orderinfo_hw"]//th[contains(text(),"WIFI与蓝牙")]/../td//strong/text()'
        wifi_bluetooth_list = self.html.xpath(wifi_bluetooth_xpath)
        wifi_bluetooth_str += "".join(wifi_bluetooth_list).strip() if wifi_bluetooth_list else ""
        wifi_bluetooth_xpath_2 = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"WIFI与蓝牙")]/../td/p/text()'
        wifi_bluetooth_list_2 = self.html.xpath(wifi_bluetooth_xpath_2)
        wifi_bluetooth_str += "".join(wifi_bluetooth_list_2[0]).strip() if wifi_bluetooth_list_2 else ""
        wifi_str = "".join(wifi_bluetooth_str).strip() if wifi_bluetooth_str else ""
        return wifi_str

    def get_choose_ci(self):
        choose_xpath = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"可选功能")]/../td//strong/text()'
        choose_list = self.html.xpath(choose_xpath)
        # CI
        ci_str = 'CI' if 'CI' in "".join(choose_list).strip() else ""
        self._logger.info("可选功能CI类型：" + str(ci_str))
        return ci_str

    def get_choose_wifi(self):
        choose_xpath = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"可选功能")]/../td/strong/text()'
        choose_list = self.html.xpath(choose_xpath)
        wifi_module_str = ""
        if '内置WIFI模块' in "".join(choose_list).strip():
            wifi_module_xpath_1 = \
                '//*[@id="orderinfo_hw"]//th[contains(text(),"可选功能")]/../td/strong/text()'
            wifi_module_list_1 = self.html.xpath(wifi_module_xpath_1)
            wifi_module_str += "".join(wifi_module_list_1).strip() if wifi_module_list_1 else ""
            wifi_module_xpath = \
                '//*[@id="orderinfo_hw"]//th[contains(text(),"可选功能")]/../td/p/text()'
            wifi_module_list = self.html.xpath(wifi_module_xpath)
            wifi_module_str += "".join(wifi_module_list[0]).strip() if wifi_module_list else ""
            self._logger.info("可选功能WIFI模块：" + wifi_module_str)
            return wifi_module_str
        else:
            return ""

    def get_customer_special_requirement(self):
        customer_special_requirement_xpath = \
            '//*[@id="orderinfo_hw"]//th[contains(text(),"客户特殊需求")]/../td//strong/text()'
        customer_special_requirement_list = self.html.xpath(customer_special_requirement_xpath)
        customer_special_requirement_str = "".join(customer_special_requirement_list).strip('\n').strip() if customer_special_requirement_list else ""
        self._logger.info("客户特殊需求：" + customer_special_requirement_str)
        return customer_special_requirement_str

    def get_request(self):
        """
        # 返回：字典
        # 包含内容（eg）：
        # {'产品型号': 'TP.MS3663T.PB707'
        # '主芯片': 'MSD3666LSAT-U9-广东海信-客供(DD;DD+;RMVB)',
        # 'FlashSize': '8MByte',
        # '子板型': 'PB707_A3',
        # 'ddrSize': '',
        # '可选功能': '无',
        # '区域': '菲律宾',
        # 'CI PLUS功能': '无',
        # '占空比': '70'
        # ...
        #
        # }
        """
        request_dict = dict()
        # 产品型号
        request_dict[self.product_name] = self.get_product_name()
        # 主芯片
        request_dict[self.chip_name] = self.get_chip_name()
        # flash size
        request_dict[self.flash_size_name] = self.get_flash_size()
        # 子板型
        request_dict[self.port_name] = self.get_port_name()
        # DDR
        request_dict[self.ddr_name] = self.get_ddr_size()
        # 可选功能
        request_dict[self.ci_type] = self.get_choose_ci()
        # WIFI模块
        request_dict[self.wifi_module_info] = self.get_choose_wifi()
        # 区域
        request_dict[self.region_name] = self.get_region_name()
        # CI Plus
        request_dict[self.ciplus_name] = self.get_ci_plus()
        # 占空比
        request_dict[self.pwm_name] = self.get_pwm()
        # 高频头
        request_dict[self.tuner_name] = self.get_tuner_type()
        # Title
        request_dict[self.title_name] = self.get_ocs_title()
        # 客户批号
        request_dict[self.customer_batch_code] = self.get_customer_batch_code()
        # 客户机型
        request_dict[self.customer_machine] = self.get_customer_machine()
        # 其它应用软件
        request_dict[self.other_app_soft] = self.get_other_app_software()
        # WIFI与蓝牙
        request_dict[self.wifi_bluetooth_type] = self.get_wifi_bluetooth()
        # 主芯片备注
        request_dict[self.chip_remark] = self.get_chip_remark()
        # 客户特殊需求
        request_dict[self.customer_special_requirement] = self.get_customer_special_requirement()

        return request_dict
