import logging
import datetime
from pathlib import Path
from lxml import etree
import mimetypes
from pyocs import pyocs_confluence
from pyocs.pyocs_filesystem import PyocsFileSystem
from pyocs.pyocs_analysis import PyocsAnalysis
from pyocs.pyocs_request import PyocsRequest
from requests_toolbelt import MultipartEncoder
from pyocs.pyocs_demand import PyocsDemand
import os
import re
import xlrd

"""
# @author:zhubowen3432
# @作用：OCS上的软件相关的功能
# @className：PyocsSoftware
"""


class PyocsSoftware:
    _instance = None
    _ocs_base_link = 'http://ocs.gz.cvte.cn'
    # _ocs_base_link = 'http://ocstest.gz.cvte.cn'
    _search_url = _ocs_base_link + '/tv/Attachments/search_firmwares'
    _ocs_link_Prefix = _ocs_base_link + '/tv/Tasks/view/'
    _refresh_url_prefix = _ocs_base_link + "/tv/Attachments/GenerateFsShortUrlByAttachmentId/"
    _master_url = _ocs_base_link + "/tv/FirmwareAttachmentRelInfos/set_fw_is_master_json/"
    _auto_bin_url_prefix = _ocs_base_link + "/tv/Tasks/auto_sw_bin_json/"  # 410430/5c275e6d-987c-4b6f-a60a-1ab6ac11527a/NULL/NULL
    _upload_fw_url_prefix = _ocs_base_link + "/tv/pop/Tasks/upload_fw_attachment/"

    _ocs_confirm_url = _ocs_base_link + "/tv/Tasks/change_fw_property_json/"
    _ocs_comment_url = _ocs_base_link + "/tv/TaskComments/save_task_comment_json/"
    _ocs_find_old_sw_url = _ocs_base_link + "/tv/pop/selectors/common_view/Controller:FirmwareAttachments/" \
                                            "Parameters:accountId/input_id:old_fw_id/input_name:old_fw_name/inputtype:1"
    sw_test_type = {'N': '100', 'A': '1', 'B': '2', 'C': '3', 'D': '4', 'E': '5', 'F': '6', 'G': '7', 'BC': '8'}
    _logger = logging.getLogger(__name__)

    def __init__(self):
        self._logger.setLevel(level=logging.WARNING)  # 控制打印级别

    # 单例模式
    def __new__(cls, *args, **kw):
        if not cls._instance:
            cls._instance = super(PyocsSoftware, cls).__new__(cls, *args, **kw)
        return cls._instance

    class SoftwareDownloadlink:
        def __init__(self):
            self.name = ''  # 软件名
            self.download_link = ''  # 软件下载链接
            self.deadline = ''  # 链接截止日期

    class Software:
        def __init__(self):
            self.name = ''  # 软件名
            self.attachment_id = ''  # 软件attachment id
            self.fw_id = ''  # 软件fw_id

    @staticmethod
    def is_burn_bin(sw_name):
        """
        检查是否是烧录bin
        :param sw_name:
        :return:
        """
        return "EMMCBIN" in sw_name or "NANDBIN" in sw_name

    @staticmethod
    def is_cvte_spec_sw(sw_name):
        """
        检查是否符合CVTE软件命名规范
        :return:
        """
        return "CP" in sw_name or "CS" in sw_name

    def get_ocs_html(self, ocs_number):
        ocs_link = self._ocs_link_Prefix + ocs_number
        response = PyocsRequest().pyocs_request_get(ocs_link)
        html = etree.HTML(response.text)
        return html

    @PyocsAnalysis.print_run_time
    def get_software_list_from_search_url_by_sw_info(self, sw_info: str):
        """从OCS搜索软件的页面（http://ocs.gz.cvte.cn/tv/Attachments/search_firmwares）搜索软件
        Args:
             sw_info: 软件名、软件版本号、或者软件名中的部分信息
        Returns:
             返回搜索到的软件(类型@Software，包含软件名和attachment id)列表，筛除烧录bin，筛除库存重复软件
        """
        software_list = list()
        sw_name_set = set()
        search_form_data = dict()
        search_form_data['_method'] = 'POST'
        search_form_data['AttachmentName'] = sw_info
        search_response = PyocsRequest().pyocs_request_post(self._search_url, data=search_form_data)
        html = etree.HTML(search_response.text)
        sw_name_str_list = html.xpath('//a[@title="下载软件"]/../../td[@colspan="4"]/text()')
        for idx, sw_name_str in enumerate(sw_name_str_list):
            if self.is_burn_bin(sw_name_str):  # 需要剔除bin包
                continue
            sw_name_set.add(sw_name_str)
        self._logger.info("软件名集合：" + str(sw_name_set))
        sw_name_list = list(sw_name_set)
        self._logger.info("软件名集合：" + str(sw_name_list))
        for sw_name in sw_name_list:
            sw = self.Software()
            attachment_xpath_str = '//td[text()="' + sw_name + '"]/..//a[@title="下载软件"]/@href'
            self._logger.info(attachment_xpath_str)
            tmp_attachment_id_list = html.xpath(attachment_xpath_str)
            self._logger.info(tmp_attachment_id_list)
            tmp_list = tmp_attachment_id_list[0].split('/')
            attachment_id = tmp_list[-1]
            sw.name = sw_name
            self._logger.info("软件名：" + sw.name)
            sw.attachment_id = attachment_id
            software_list.append(sw)
        return software_list

    def get_refresh_software_download_link_by_attachment_id(self, attachment_id):
        """根据输入的软件的attachment_id 刷新软件下载链接
        Args:
            attachment_id: 软件的attachment_id
        Returns:
            返回刷新后的下载链接和下载链接有效期
        """
        refresh_url = self._refresh_url_prefix + attachment_id
        response = PyocsRequest().pyocs_request_get(refresh_url)
        ret_dict = response.json()
        download_link = str(ret_dict['datas'])
        self._logger.info("刷新后的下载链接:" + str(download_link))
        deadline = (datetime.datetime.now() + datetime.timedelta(days=30)).strftime('%Y-%m-%d %H:%M:%S')
        return download_link, deadline

    def get_refresh_software_download_link_by_sw_info(self, sw_info):
        """根据软件局部信息更新软件外网下载链接和链接更新日期
        Args:
            sw_info: 软件名、软件版本号、或者软件名中的部分信息
        Returns:
            返回软件刷新后的下载链接和下载链接有效期
        """
        sdl_list = list()
        sw_list = self.get_software_list_from_search_url_by_sw_info(sw_info)
        for sw in sw_list:
            sdl = self.SoftwareDownloadlink()
            sdl.download_link, sdl.deadline = self.get_refresh_software_download_link_by_attachment_id(sw.attachment_id)
            sdl.name = sw.name
            sdl_list.append(sdl)
        return sdl_list

    def get_download_link_from_html_by_attachment_id(self, html, attachment_id):
        """根据attachment_id获取软件外网下载链接和链接更新日期
        Args:
            html: 某个ocs订单的html页面
            attachment_id: 软件的attachment id
        Returns:
            返回软件下载链接和下载链接有效期
        """
        url_xpath_str = '//a[@attachment_id="' + attachment_id + '"]/@es_url'
        download_link_list = html.xpath(url_xpath_str)
        update_time_xpath_str = '//a[@attachment_id="' + attachment_id + '"]/@es_update_time'
        update_time_list = html.xpath(update_time_xpath_str)
        self._logger.info("get_download_link_from_html_by_attachment_id 下载链接：" + str(download_link_list))
        update_time = datetime.datetime.strptime(update_time_list[0], "%Y-%m-%d %H:%M:%S")
        deadline = (update_time + datetime.timedelta(days=30)).strftime('%Y-%m-%d %H:%M:%S')
        return download_link_list[0], deadline

    def get_download_link_by_software_name(self, sw_name):
        """根据软件全名获取软件外网下载链接和链接更新日期
        Args:
            sw_name: 软件的全名
        Returns:
            返回软件下载链接和下载链接有效期
        """
        sdl = self.SoftwareDownloadlink()
        split_str = sw_name.split('_')
        ocs_number = split_str[0].lstrip('CP').lstrip('CS')
        ocs_url = self._ocs_link_Prefix + ocs_number
        ocs_response = PyocsRequest().pyocs_request_get(ocs_url)
        html = etree.HTML(ocs_response.text)
        attachment_id = self.get_enable_software_attachment_id_from_html_by_sw_name(html=html, sw_name=sw_name)
        sdl.download_link, sdl.deadline = self.get_download_link_from_html_by_attachment_id(html=html,
                                                                                            attachment_id=attachment_id)
        sdl.name = sw_name
        return sdl

    def get_software_download_link(self, ocs_number, sw_name):
        """根据软件全名获取软件外网下载链接和链接更新日期
        Args:
            ocs_number:订单号
            sw_name: 软件的全名
        Returns:
            返回软件下载链接和下载链接有效期
        """
        sdl = self.SoftwareDownloadlink()
        ocs_url = self._ocs_link_Prefix + ocs_number
        ocs_response = PyocsRequest().pyocs_request_get(ocs_url)
        html = etree.HTML(ocs_response.text)
        attachment_id = self.get_enable_software_attachment_id_from_html_by_sw_name(html=html, sw_name=sw_name)
        sdl.download_link, sdl.deadline = self.get_download_link_from_html_by_attachment_id(html=html,
                                                                                            attachment_id=attachment_id)
        sdl.name = sw_name
        return sdl

    def get_enable_software_list_from_html(self, html):
        """获取订单上的启用状态下的软件列表
        Args:
            html: 某个ocs订单的html页面
        Returns:
            启用状态下的软件列表
        """
        enable_software_list = []
        # enable_software_name_xpath = '//strong[text()="同时停用以下软件包"]/../../..//input[@type="checkbox"]/..//label/text()'  # '//a[@title="停用软件"]/@file_name'
        enable_software_name_xpath = \
            '//a[contains(@href, "/tv/Attachments/download_attachment")]/text()'
        enable_software_name_list = html.xpath(enable_software_name_xpath)
        self._logger.info("启用状态的软件:" + str(enable_software_name_list))
        for sw_name in enable_software_name_list:
            if not self.is_sw_locked(sw_name=sw_name, html=html):
                enable_software_list.append(sw_name)
        return enable_software_list

    @staticmethod
    def is_sw_locked(sw_name, html):
        tmp_xpath = '//strong[text()="软件包"]/../../..//a[text()="' + sw_name + '"]/../a[@data-locktype="LOCK"]/i/@class'
        tmp_list = html.xpath(tmp_xpath)
        is_locked = True if 'icon-eye-close' in tmp_list else False
        return is_locked

    def get_all_software_list_from_html(self, html):
        """获取订单上的所有软件
        Args:
            html: 某个ocs订单的html页面
        Returns:
            所有软件，以列表返回
        """
        sw_name_list = html.xpath('//div[@id="file_names"]/p/text()')
        self._logger.info("软件名列表：" + str(sw_name_list))
        return sw_name_list

    def get_enable_software_attachment_id_from_html_by_sw_name(self, html, sw_name):
        """获取订单上的特定启用状态下软件的attachment_id，非启用状态的软件，返回为空
        Args:
            html: 某个ocs订单的html页面
            sw_name: 需要获取attachment_id的软件名（软件全名）
        Returns:
            软件的attachment id，没有找到则返回为空
        """
        attachment_id_xpath = '//a[text()="' + sw_name + '"]/@href'
        attachment_id_str_list = html.xpath(attachment_id_xpath)
        self._logger.info("attachment_id_str_list:" + str(attachment_id_str_list))
        if attachment_id_str_list:
            attachment_id_split_list = attachment_id_str_list[0].split('/')
            attachment_id = attachment_id_split_list[-1]
            self._logger.info("attachment id:" + attachment_id)
            return attachment_id
        else:
            return None

    def get_enable_software_fw_id_from_html_by_sw_name(self, html, sw_name):
        """获取订单上的特定启用状态下软件的fw_id，非启用状态的软件，返回为空
        Args:
            html: 某个ocs订单的html页面
            sw_name: 需要获取fw id的软件名（软件全名）
        Returns:
            软件的fw id，没有找到则返回为空
        """
        fw_id_xpath = '//a[text()="' + sw_name + '"]/../span/text()'
        fw_id_str_list = html.xpath(fw_id_xpath)
        if fw_id_str_list:
            fw_id = fw_id_str_list[0].strip()
            self._logger.info("fw_id:" + fw_id)
            return fw_id
        else:
            return None

    @PyocsAnalysis.print_run_time
    def get_enable_software_list(self, ocs_number, exclude_bin=True):
        """
        获取订单中启用状态下的软件列表，以Software对象列表返回
        """
        sw_list = list()
        ocs_link = self._ocs_link_Prefix + ocs_number
        response = PyocsRequest().pyocs_request_get(ocs_link)
        html = etree.HTML(response.text)
        sw_name_list = self.get_enable_software_list_from_html(html)
        for sw_name in sw_name_list:
            sw = self.Software()
            if exclude_bin and self.is_burn_bin(sw_name=sw_name):
                continue
            if exclude_bin and not self.is_cvte_spec_sw(sw_name=sw_name):
                continue
            sw.name = sw_name
            sw.attachment_id = self.get_enable_software_attachment_id_from_html_by_sw_name(html, sw_name)
            sw.fw_id = self.get_enable_software_fw_id_from_html_by_sw_name(html, sw_name)
            sw_list.append(sw)
        self._logger.info("软件名和attchment id的字典：" + str(sw_list))
        if sw_list:
            return sw_list
        else:
            return None

    @PyocsAnalysis.print_run_time
    def set_master_sw(self, fw_id):
        """根据fw_id，设置主程序，常用于做bin
        """
        self._logger.info('fw_id：' + str(fw_id))
        data = {
            'data[fw_id]': int(fw_id),
            'data[action]': 'master',
            'data[is_master]': 1
        }
        ret = PyocsRequest().pyocs_request_post(self._master_url, data=data)
        self._logger.info(ret.text)
        ret_dict = ret.json()
        self._logger.info(ret_dict['success'])
        if ret_dict['success'] is True:
            return True
        else:
            return False

    def auto_make_bin(self, ocs_number, attachment_id):
        _auto_bin_url = self._auto_bin_url_prefix + ocs_number + '/' + attachment_id
        response = PyocsRequest().pyocs_request_post(_auto_bin_url, data=None)
        ret_dict = response.json()
        self._logger.info(ret_dict['success'])
        if ret_dict['success'] is True:
            return True
        else:
            return False

    def auto_make_bin_by_sw_name(self, sw_name: str):
        ocs_number_str = re.search("C[PS][0-9]*", sw_name).group(0).lstrip('CP').lstrip('CS')
        ocs_link = self._ocs_link_Prefix + ocs_number_str
        response = PyocsRequest().pyocs_request_get(ocs_link)
        html = etree.HTML(response.text)
        sw_attachment_id = self.get_enable_software_attachment_id_from_html_by_sw_name(html, sw_name)
        sw_fw_id = self.get_enable_software_fw_id_from_html_by_sw_name(html, sw_name)
        self.set_master_sw(sw_fw_id)
        return self.auto_make_bin(ocs_number_str, sw_attachment_id)

    def add_comment_to_ocs(self, ocs_number, message):

        strlist = message.split('\n')
        comment = ""
        for value in strlist:
            comment = comment + '<p style="margin: 2px;">' + value + "</p>"

        data = {
            'task_id': int(ocs_number),
            'comment': comment,
            'comment_text': str(message),
            'parent_id': 0,
            'url': " "
        }

        ret = PyocsRequest().pyocs_request_post(self._ocs_comment_url, data=data)
        self._logger.info("ocs 备注message：" + comment)
        return True

    """
       # 根据订单号特性转换
    """

    def order_info_transfer(slef, order_info):
        # 可能出现在摘要中的订单号 X19026 X-19026 X219026 X2-19026
        order_info_list = [order_info]
        order_sign = "-"
        order_type = order_info[:-5]
        order_number = order_info[-5:]
        order_info_list.append(order_type + order_sign + order_number)  # CHAOYE X-19026形式
        return order_info_list

    """
    # 根据sw_name和order_info，获取对应OCS号
    """

    @PyocsAnalysis.print_run_time
    def get_ocs_number_from_sw(self, sw_name, order_info):
        ocs_number = str()
        search_form_data = dict()
        search_form_data['_method'] = 'POST'
        search_form_data['AttachmentName'] = sw_name
        search_response = PyocsRequest().pyocs_request_post(self._search_url, data=search_form_data)
        html = etree.HTML(search_response.text)
        str_ocs_number_xpath = '//strong[@class="label"]/text()'
        ocs_number_list = html.xpath(str_ocs_number_xpath)
        order_info_list = self.order_info_transfer(order_info)
        for ocs_number_index in set(ocs_number_list):
            tmp_ocs_number = ocs_number_index.strip("#")
            str_ocs_abstract_xpath = '//a[contains(@href,"/tv/Tasks/view/' + tmp_ocs_number + '")]/text()'
            ocs_abstract_info = html.xpath(str_ocs_abstract_xpath)
            for order_info_index in order_info_list:
                if order_info_index in str(ocs_abstract_info):
                    ocs_number = tmp_ocs_number
        if ocs_number:
            return ocs_number
        else:
            self._logger.info("ocs_number not found")

    def reuse_old_sw_from_src_to_dst(self, src_ocs, dst_ocs, workspace):
        """
        引用库存软件，如果此单上已经引用了同名的库存软件，则不做引用
        :param src_ocs:
        :param dst_ocs:
        :return:
        """
        dst_ocs_sw_name_list = []
        reuse_sw_name_list = []
        dst_ocs_sw_list = self.get_enable_software_list(ocs_number=dst_ocs, exclude_bin=False)
        if dst_ocs_sw_list:
            for dst_ocs_sw in dst_ocs_sw_list:
                # OCS系统的逆天bug，可能会出现两单相同软件，但是其中一个字母一边大写一边小写，此处为了这个bug，统一采用大写比较
                dst_ocs_sw_name_list.append(str(dst_ocs_sw.name.upper()))
        sw_list = self.get_enable_software_list(ocs_number=src_ocs, exclude_bin=False)
        for index, sw in enumerate(reversed(sw_list)):
            if str(sw.name).upper() not in dst_ocs_sw_name_list:
                self.upload_old_sw_by_id(ocs_num=dst_ocs, old_sw_id=sw.fw_id, disable_origin_sw=(index == 0))
                reuse_sw_name_list.append(sw.name)
        while reuse_sw_name_list[-1] \
                not in self.get_enable_software_list_from_html(PyocsDemand(ocs_number=dst_ocs).get_ocs_html()):
            continue
        else:
            pd = PyocsDemand(ocs_number=dst_ocs)
            customer = pd.get_ocs_customer()
            engineer = pd.get_ocs_software_engineer()
            if customer in ["TCL海外", "TCL王牌"] and "D" in engineer:
                kb = pyocs_confluence.PyocsConfluence()
                download_link = kb.get_download_link_of_tcl_dt_by_engineer(pd.get_ocs_software_engineer())
                excel_file_location = PyocsFileSystem.get_file_from_nut_driver(url=download_link, workspace=workspace)
                for sw_name in reuse_sw_name_list:
                    if not ("D赵壮洪" in engineer and "TCL王牌" in customer):
                        self.upload_tcl_special_excel(ocs_num=dst_ocs, sw_name=sw_name,
                                                      excel_file_location=excel_file_location, pd=pd)
                    if "TCL王牌" in customer and "D赵壮洪" not in engineer:
                        self.set_software_confirm_for_ocs(ocs_number=dst_ocs, sw_name=sw_name)
                if "TCL王牌" in customer and "D赵壮洪" not in engineer:
                    self.comment_tcl_software_info(excel_file_location=excel_file_location,
                                                   customer_batch=pd.get_customer_batch_code(), ocs_number=dst_ocs)

    def comment_tcl_software_info(self, excel_file_location, customer_batch, ocs_number):
        workbook = xlrd.open_workbook(filename=excel_file_location)
        table = workbook.sheet_by_index(0)
        batch_code_list = table.col_values(0)
        row_index = None
        row_numbers = table.nrows
        for index, col_value in enumerate(reversed(batch_code_list)):
            if col_value.strip() == customer_batch:
                row_index = row_numbers - index - 1
                break
        html_table = '<table border="1">'
        header = '<tr>'
        message = '<tr>'
        for index, info in enumerate(table.row_values(row_index)):
            if info:
                header += '<td>' + table.cell(0, index).value + '</td>'
                info_str = str(int(info)) if type(info) == float else str(info)
                message += '<td>' + info_str + '</td>'
        message += '</tr>'
        header += '</tr>'
        html_table += header + message + '</table>'
        # print(html_table)
        self.add_comment_to_ocs(ocs_number=ocs_number, message=html_table)

    def upload_tcl_special_excel(self, ocs_num, sw_name, excel_file_location, pd: PyocsDemand):
        headers = dict()
        compare_url = 'http://ocs.gz.cvte.cn/tv/pop/Tasks/compare_request/' + ocs_num + '/'
        html = pd.get_ocs_html()
        fw_id = self.get_enable_software_fw_id_from_html_by_sw_name(html=html, sw_name=sw_name)
        attachment_id = self.get_enable_software_attachment_id_from_html_by_sw_name(html=html, sw_name=sw_name)
        print(attachment_id)
        compare_url += attachment_id
        upload_url = 'http://ocs.gz.cvte.cn/Attachments/upload_attachment_json/63/' + \
                     fw_id + "/230/xmlexcel"
        headers.update({
            'Referer': compare_url
        })
        files = {'attach_file': ("tcl_xml_excel.xls", open(excel_file_location, 'rb'),
                                 '%s' % self.get_content_type(excel_file_location))}
        ret = PyocsRequest().pyocs_request_post_file(upload_url, file=files, header=headers)
        if ret.json()['success']:
            return True
        else:
            raise RuntimeError("异常信息：" + ret.json()['message'])

    @staticmethod
    def get_content_type(filepath):
        return mimetypes.guess_type(filepath)[0] or 'application/octet-stream'

    def set_software_confirm(self, sw_name, order_info, confirm_type=4):
        """
        # 根据sw_name和order_info，确认软件
        """
        ocs_number = self.get_ocs_number_from_sw(sw_name, order_info)
        if ocs_number is None:
            pass
        else:
            sw_list = self.get_enable_software_list(ocs_number=ocs_number)
            fw_id = sw_list[0].fw_id
            confirm_status = confirm_type  # 1-未确认，3-口头确认，(default)4-邮件确认，5-不需确认
            data = {
                'task_id': int(ocs_number),
                'field': "account_firmware_status",
                'firmware_id': int(fw_id),
                'data': confirm_status,
                'confirm_reason': 1,
                'first_confirm_reason': str(ocs_number),
                'second_confirm_reason': str(sw_name)
            }
            ret = PyocsRequest().pyocs_request_post(self._ocs_confirm_url, data=data)
            confirm_result = ret.text
            self._logger.info(confirm_result)
            if int(confirm_result) > 0:
                return True
            else:
                self._logger.info("confirm failed")

    def set_software_confirm_for_ocs(self, ocs_number: str, sw_name: str, confirm_type=4):
        """
            确认软件
        :param ocs_number:
        :param sw_name:
        :param confirm_type: 1-未确认，3-口头确认，(default)4-邮件确认，5-不需确认
        :return:
        """
        html = self.get_ocs_html(ocs_number=ocs_number)
        fw_id = self.get_enable_software_fw_id_from_html_by_sw_name(html=html, sw_name=sw_name)
        data = {
            'task_id': int(ocs_number),
            'field': "account_firmware_status",
            'firmware_id': int(fw_id),
            'data': confirm_type,
            'confirm_reason': 1,
            'first_confirm_reason': str(ocs_number),
            'second_confirm_reason': str(sw_name)
        }
        ret = PyocsRequest().pyocs_request_post(self._ocs_confirm_url, data=data)
        return ret.status_code == 200

    def get_test_type_from_ocs(self, ocs_number: str):
        """
            获取OCS软件的测试类型
        :param ocs_number:
        :return:test type str
        """
        test_type_xpath = '//*[@id="current_test_type_text"]/text()'
        html = self.get_ocs_html(ocs_number=ocs_number)
        test_type_list = html.xpath(test_type_xpath)
        self._logger.info(test_type_list)
        if len(test_type_list) < 1:
            return None
        else:
            test_type_str = str(test_type_list[0]).replace(" ", "").replace("\n", "").replace("\r", "")
            return test_type_str

    def get_enable_sw_audit_link(self, ocs_number: str):
        """
            获取OCS里使能软件自动审核不通过的审核页链接
        :param ocs_number:
        :return:test type str
        """
        #audit_link_xpath = '//a[@title="自动审核失败" or @title="自动审核通过" ]/./@onclick'
        #audit_link_xpath = '//a[@title="自动审核失败"]/./@onclick'
        audit_link_xpath = '//a[@title="停用软件"]/../a[@title="自动审核失败"]/./@onclick'
        html = self.get_ocs_html(ocs_number=ocs_number)
        tmp_link = html.xpath(audit_link_xpath)
        if len(tmp_link) < 1:
            return None
        else:
            link_str = str(tmp_link).split(",")[0].split("'")[-2].rstrip("\\")
            return link_str

    def get_self_audit_msg(self, audit_link_str: str):
        """
            获取OCS软件审核页的不匹配信息
        :param audit_link_str:
        :return:self audit not pass msg
        """
        audit_pass_xpath = '//tr[@class="row_impoarant class_ab_row " \
                            or @class="row_impoarant class_ab_row row_compare-diff"]/th/text()'
        if audit_link_str is None:
            return None
        else:
            audit_link = self._ocs_base_link + audit_link_str
            self._logger.info(audit_link)
            response = PyocsRequest().pyocs_request_get(audit_link)
            html = etree.HTML(response.text)
            audit_msg_ret = html.xpath(audit_pass_xpath)
            return audit_msg_ret

    def find_old_sw_id_by_name(self, old_sw_name):
        """根据 old_sw_name 查找库存软件 old_sw_id
        Args:
            old_sw_name: 软件名
        Returns:
            查找到的软件字典
        """
        old_sw_dict = dict()
        self._logger.info(old_sw_name)
        data = {
            'data[autocomplete]': str(old_sw_name)
        }
        search_response = PyocsRequest().pyocs_request_post(self._ocs_find_old_sw_url, data=data, allow_redirects=True)
        html = etree.HTML(search_response.text)
        old_sw_id_str_list = html.xpath('//div[@class="clearfix"]//input/@value')
        sw_name_list = html.xpath('//td[@class="FirmwareAttachment_col_attachment_id  MainField"]/text()')
        if old_sw_id_str_list and sw_name_list:
            old_sw_dict = dict(zip(old_sw_id_str_list, sw_name_list))
        self._logger.info("ocs 库存软件查找：" + str(old_sw_dict))
        return old_sw_dict

    def upload_old_sw_by_id(self, ocs_num, old_sw_id, disable_origin_sw=True):
        """根据ocs_num 和old_sw_name， 上传库存软件
        """
        fields = {'data[fw_Attach][file][]': '(binary)',
                  'data[fw_Attach][old_fw_id][]': str(old_sw_id),
                  'data[fw_Attach][ab_sw_name][]': '',
                  'data[fw_Attach][ab_sw_id][]': '',
                  'data[fw_Attach][ab_xml_name][]': '',
                  'data[fw_Attach][ab_xml_id][]': '',
                  'data[fw_Attach][autobuild_bill_no][]': '',
                  'data[fw_Attach][fileXML][]': '(binary)',
                  'data[fw_Attach][test_type][]': "100",  # 默认不用测试
                  'data[fw_Attach][fw_cfm_type]': '',
                  'data[fw_Attach][soft_remark]': '',
                  'data[fw_Attach][function_description]': '',
                  'data[fw_Attach][result_of_modified]': '',
                  'data[fw_Attach][note][]': "引用库存软件",
                  'data[change_task_status]': 'on'
                  }
        if disable_origin_sw:
            sw_list = self.get_enable_software_list(ocs_number=ocs_num, exclude_bin=False)
            if sw_list:
                for sw in sw_list:
                    fields.update({'data[fw_Attach][fileList][' + sw.attachment_id + ']': 'on'})
        ret = PyocsRequest().pyocs_request_post(self._upload_fw_url_prefix + str(ocs_num), data=fields)
        return ret.status_code == 200

    def upload_software_to_ocs(self, ocs_num, zip_path, xml_path, test_type, message: str):
        """根据ocs_num 和sw_path， 上传软件
        Args:
            ocs_num: 订单号
            zip_path: 软件包地址
            xml_path: xml地址
            test_type:
                0 未知
                1 A类 - 全面测试
                2 B类 - 针对遥控器的测试
                3 C类 - 局部修改针对性测试
                4 D类 - 测配屏
                5 E类 - 首测，无任何功能及BUG的修改
                6 F类 - 过硬件首测，只要求匹配硬件
                7 G类 - 仅测试BOOT
                8 B+C - 新遥控器测试，局部功能修改/添加
                100 不用测试
            message: 上传软件备注

        Returns:
            成功则返回为True
        """
        headers = dict()
        data_fields = dict()
        test_type_set = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '100'}
        if test_type not in test_type_set:
            return False
        data_fields.update({
            'data[fw_Attach][file][]': (os.path.basename(zip_path), open(zip_path, 'rb'), 'application/zip'),
            'data[fw_Attach][old_fw_id][]': '',
            'data[fw_Attach][ab_sw_name][]': '',
            'data[fw_Attach][ab_sw_id][]': '',
            'data[fw_Attach][ab_xml_name][]': '',
            'data[fw_Attach][ab_xml_id][]': '',
            'data[fw_Attach][autobuild_bill_no][]': '',
            'data[fw_Attach][fileXML][]': (
                os.path.basename(xml_path), open(xml_path, 'rb'), 'application/octet-stream'),
            'data[fw_Attach][test_type][]': test_type,
            'data[fw_Attach][fw_cfm_type]': '0',
            'data[fw_Attach][soft_remark]': '',
            'data[fw_Attach][function_description]': '',
            'data[fw_Attach][result_of_modified]': '',
            'data[fw_Attach][note][]': message,
            'data[change_task_status]': 'on'
        }
        )

        sw_list = self.get_enable_software_list(ocs_number=ocs_num, exclude_bin=False)
        if sw_list:
            for sw in sw_list:
                data_fields.update({'data[fw_Attach][fileList][' + sw.attachment_id + ']': 'on'})

        data = MultipartEncoder(
            fields=data_fields
        )

        headers.update({
            'Referer': self._upload_fw_url_prefix + ocs_num
        })
        headers['Content-Type'] = data.content_type
        self._logger.info(headers)
        ret = PyocsRequest().pyocs_request_post_with_headers(self._upload_fw_url_prefix + ocs_num,
                                                             data=data, headers=headers)
        self._logger.info(ret.text)
        return ret.status_code == 200

    def comment_factory_test_info(self, ocs_num: str, message: str):
        factory_test_url_pre = "http://ocs.gz.cvte.cn/Tasks/add_task_test_remark/"
        factory_test_url = factory_test_url_pre + ocs_num
        message_list = message.split(";")
        data_fields = dict()
        for idx, mess in enumerate(message_list):
            data_fields.update({'data[TestRemark' + str(idx + 1) + ']': mess})
        # print(data_fields)
        ret = PyocsRequest().pyocs_request_post(factory_test_url, data=data_fields)
        return ret.status_code == 302
